/*
 * DESCRIPTION:
 * Mouse functions.
*/

#include "mouse.h"

/* Updates the mouse. */
void
mouse_update(Mouse* self)
{
        s32 state;
	s32 x;
	s32 y;

        memcpy(&self->previous, &self->current, sizeof(bool) * MOUSE_BUTTON_COUNT);
        memset(&self->current, '\0', sizeof(bool) * MOUSE_BUTTON_COUNT);

        state = SDL_GetMouseState(NULL, NULL);

        if ((state & SDL_BUTTON_LMASK) != 0)
                self->current[MOUSE_BUTTON_LEFT] = true;

        if ((state & SDL_BUTTON_MMASK) != 0)
                self->current[MOUSE_BUTTON_MIDDLE] = true;

        if ((state & SDL_BUTTON_RMASK) != 0)
                self->current[MOUSE_BUTTON_RIGHT] = true;

        if ((state & SDL_BUTTON_X1) != 0)
                self->current[MOUSE_BUTTON_X1] = true;

        if ((state & SDL_BUTTON_X2) != 0)
                self->current[MOUSE_BUTTON_X2] = true;

	SDL_GetMouseState(&x, &y);

	self->position[0] = x;
	self->position[1] = y;
}

/* Is a given mouse button pressed? */
bool
mouse_press(Mouse* self, MouseButtonType type)
{
        return (self->current[type] && !self->previous[type]);
}

/* Is a given mouse button held? */
bool
mouse_held(Mouse* self, MouseButtonType type)
{
        return (self->current[type] && self->previous[type]);
}

/* Is a given mouse button released? */
bool
mouse_release(Mouse* self, MouseButtonType type)
{
        return (!self->current[type] && self->previous[type]);
}
