#include "component_game_object.h"

/* DEPENDENCIES: Sprite, Physics */

/* Sets game_object component info. */
/* This assumes a one-frame sprite. */
void
component_game_object_init
(
	ComponentGameObject* self,
	ECS* ecs,
	Texture texture,
	ShaderType shader,
	OriginType origin,
	vec2 size,
	vec3 position,
	vec4 color,
	f32 friction,
	f32 velocityMax
)
{
	ivec2 atlasSize;
	ivec2 frameSize;

	frameSize[0] = (s32)texture.size[0];
	frameSize[1] = (s32)texture.size[1];

	self->angle = 0.0f;

	glm_ivec2_one(atlasSize);

	component_game_object_atlas_init
	(
		self,
		ecs,
		texture,
		shader,
		origin,
		frameSize,
		atlasSize,
		size,
		position,
		color,
		friction,
		velocityMax
	);
}

/* Sets game_object component info, with an atlas. */
void
component_game_object_atlas_init
(
	ComponentGameObject* self,
	ECS* ecs,
	Texture texture,
	ShaderType shader,
	OriginType origin,
	ivec2 frameSize,
	ivec2 atlasSize,
	vec2 size,
	vec3 position,
	vec4 color,
	f32 friction,
	f32 velocityMax
)
{
	ComponentSprite* sprite;
	ComponentPhysics* physics;
	
	sprite = ecs_component_get(ecs, ECS_COMPONENT_SPRITE, self->id);
	physics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->id);

	component_sprite_atlas_init(sprite, texture, shader, origin, frameSize, atlasSize, size, position, color);
	component_physics_init(physics, position, velocityMax, friction);
}

/* Runs on game object add. */
void
component_game_object_add(ComponentGameObject* self, ECS* ecs)
{
	ComponentSprite* sprite;

	ecs_component_add(ecs, ECS_COMPONENT_PHYSICS, self->id);
	sprite = ecs_component_add(ecs, ECS_COMPONENT_SPRITE, self->id);

	sprite = ecs_component_get(ecs, ECS_COMPONENT_SPRITE, self->id);
}

/* Runs on game object add. */
void
component_game_object_delete(ComponentGameObject* self, ECS* ecs)
{
	ecs_component_delete(ecs, ECS_COMPONENT_SPRITE, self->id);
	ecs_component_delete(ecs, ECS_COMPONENT_PHYSICS, self->id);
}

/* Ticks game_object component. */
void
component_game_object_tick(ComponentGameObject* self, ECS* ecs)
{
	ComponentSprite* sprite;
	ComponentPhysics* physics;

	sprite = ecs_component_get(ecs, ECS_COMPONENT_SPRITE, self->id);
	physics = ecs_component_get(ecs, ECS_COMPONENT_PHYSICS, self->id);

	glm_vec3_copy(physics->position, sprite->position);
}
