#include "component_sprite.h"

/* Returns sprite rectangle. */
void
component_sprite_rectangle_get(ComponentSprite* self, vec4 rectangle)
{
	vec2 size;
	vec2 origin;
	vec2 difference;

	component_sprite_size_get(self, size);

	rectangle[0] = self->position[0];
	rectangle[1] = self->position[1];
	rectangle[2] = self->position[0] + size[0];
	rectangle[3] = self->position[1] + size[1];
}

/* Sets sprite component info, additionally given atlas rows/columns. */
void
component_sprite_atlas_init
(
	ComponentSprite* self, 
	Texture texture, 
	ShaderType shader,
	OriginType origin,
	ivec2 frameSize,
	ivec2 atlasSize,
	vec2 size, 
	vec3 position,
	vec4 color
)
{
	self->shader = shader;
	self->origin = origin;

	atlas_init(&self->atlas, texture, frameSize, atlasSize);

	glm_vec2_copy((f32*)size, self->size);
	glm_vec3_copy((f32*)position, self->position);
	glm_vec4_copy((f32*)color, self->color);
}

/* Sets sprite component info. */
void
component_sprite_init
(
	ComponentSprite* self, 
	Texture texture, 
	ShaderType shader, 
	OriginType origin, 
	vec2 size, 
	vec3 position,
	vec4 color
)
{
	ivec2 atlasSize;

	glm_ivec2_one(atlasSize);

	component_sprite_atlas_init
	(
		self, 
		texture, 
		shader, 
		origin,
		texture.size,
		atlasSize,
		size,
		position,
		color
	);
}

/* Gets the true size of a sprite, accounting for scaling. */
void
component_sprite_size_get(ComponentSprite* self, vec2 size)
{
	glm_vec2_copy(self->size, size);
	glm_vec2_scale(size, self->scale, size);
}

/* Ticks sprite component. */
void
component_sprite_add(ComponentSprite* self, ECS* ecs)
{
	self->scale = COMPONENT_SPRITE_SCALE_DEFAULT;
}

/* Ticks sprite component. */
void
component_sprite_tick(ComponentSprite* self, ECS* ecs)
{
	vec4 rectangle;

	component_sprite_rectangle_get(self, rectangle);
}

/* Draws sprite component. */
void
component_sprite_draw(ComponentSprite* self, ECS* ecs)
{
	mat4 model;
	vec2 size;
	vec2 origin;
	vec3 position;
	vec3 modelOrigin;

	component_sprite_size_get(self, size);
	
	glm_vec3_copy(self->position, position);

	modelOrigin[0] = 0.0f;
	modelOrigin[1] = 0.0f;
	modelOrigin[2] = 0.0f;

	glm_mat4_identity(model);
	glm_translate(model, position);
	glm_rotate_at(model, modelOrigin, self->rotation, (f32*)COMPONENT_SPRITE_ROTATION_AXIS);

	texture_quad_draw
	(
		&ecs->game->renderer, 
		&self->atlas, 
		&ecs->game->resources.shaders[self->shader], 
		model, 
		size, 
		self->color
	);
}
